import logging

from fedora_messaging import config as fm_config

from . import config
from . import exceptions as exc
from . import senders


fm_config.conf.setup_logging()
_log = logging.getLogger(__name__)


def message_to_payload(message):
    """ Extract most of the overlapping information and let the
    sub-class complement it if they want.
    """

    if "version" not in message.body:
        data = {
            "build_id": message.body["build_id"],
            "build_url": message.body["build_url"],
            "branch": message.body["branch"],
            "ref": message.body["ref"],
            "rev": message.body["rev"],
            "repo": message.body["repo"],
            "namespace": message.body["namespace"],
            "username": message.body["username"],
            "status": message.body["status"],
        }
        for key in ["package_url", "compose_url", "nvr", "original_spec_nvr"]:
            if key in message.body:
                data[key] = message.body[key]
    # All CI messages of versions 0.2.X should have the same fields...
    elif message.body["version"].startswith("0.2."):
        # ignore scratch builds
        if message.body["artifact"]["scratch"]:
            return None
        data = {
            "build_url": message.body["run"]["url"],
            "repo": message.body["artifact"]["component"],
            "nvr": message.body["artifact"]["nvr"],
            "item": message.body["artifact"]["nvr"],
            "scratch": message.body["artifact"]["scratch"],
            "username": message.body["artifact"]["issuer"],
            "koji_task_id": message.body["artifact"]["id"],
            "namespace": message.body["test"]["namespace"],
            "type": message.body["artifact"]["type"].replace("-", "_"),
            "test_type": message.body["test"]["type"],
            "category": message.body["test"]["category"],
        }

        # pipeline.build field is optional in 0.2.X messages
        if "build" in message.body["pipeline"]:
            data["build_id"] = message.body["pipeline"]["build"]

        if "result" in message.body["test"]:
            data["status"] = message.body["test"]["result"]
    else:
        raise exc.ResultsdbListenerException(
            "Invalid CI message version: " + message.body["version"]
        )

    data["topic"] = message.topic
    data["msg_id"] = message.id

    return data


class Consumer:
    """All resultsdb-listener messages are received by this class's
    __call__() method.
    """

    def __init__(self):
        """ Initialize the consumer, sets up processor.
        """
        try:
            processor = senders.get_sender(config.sender_type)
            self.processor = processor()
        except exc.ListenerMsgError as e:
            _log.error(e)
            raise

    def __call__(self, message):
        """ React on the messages seen on fedora-messaging's bus whose topics
        are listed/defined in fedora-messaging's configuration file.

        In this case, the function will convert the message into a payload that
        will be uploaded to resultsdb.

        There is not need to take any special precaution for duplicated
        messages, we can just upload them twice if that happens.
        """

        _log.info(
            "PipelineConsumerFedoraMessaging: Got a message %r: %r",
            message.topic,
            message.id,
        )

        try:
            data = message_to_payload(message)
            if data:
                self.processor.send(data)
        except exc.ResultsdbListenerException as e:
            _log.error(e)

        _log.info("PipelineConsumerFedoraMessaging: Done")
